import argparse
from merger import merge

parser = argparse.ArgumentParser('Selectively merge cells from two excels based on a common column')
parser.add_argument('-ff', '--first-file', help='First file to merge', required=True)
parser.add_argument('-fs', '--first-sheet', help='Sheet on the first file', default='Sheet1')
parser.add_argument('-sf', '--second-file', help='Second file to merge', required=True)
parser.add_argument('-ss', '--second-sheet', help='Sheet on the second file', default='Sheet1')
parser.add_argument('--save-as', help='Save the output to a different file name', default=None)
parser.add_argument('-j', '--join-column',
                     help='Header of the column to use for the merge, must be the same in both files')
parser.add_argument('-m', '--merge-cell', help='Header of the columns to merge', action='append')
parser.add_argument('-v', '--verbose', help='Print logs to console', action='store_true', default=False)
parser.add_argument('--dry-run', help='Do not save', action='store_true', default=False)
parser.add_argument('--max-empty-rows',
                    help='Number of empty rows to encounter before considering file finished',
                    type=int, default=10)
parser.add_argument('--max-empty-columns',
                    help='Number of empty columns to encounter before considering file finished',
                    type=int, default=10)
parser.add_argument('--allow-override',
                    help='Override cells in the destination file if they are not empty',
                    action='store_true', default=False)
parser.add_argument('--highlight',
                    help='Color cells written by this script',
                    action='store_true', default=False)
parser.add_argument('--debug', help='Print ALL logs to console', action='store_true', default=False)

args = parser.parse_args()
merge(
    args.first_file,
    args.second_file,
    args.first_sheet,
    args.second_sheet,
    args.join_column,
    args.merge_cell,
    max_empty_columns=args.max_empty_columns,
    max_empty_rows=args.max_empty_rows,
    save_as=args.save_as,
    verbose=args.verbose,
    dry_run=args.dry_run,
    override=args.allow_override,
    debug=args.debug,
    highlight=args.highlight
)
