import openpyxl


class Side:
    def __init__(self, name, join_column, merge_cell, max_empty_rows, max_empty_columns, verbose, debug):
        self.name = name
        self.workbook = None
        self.sheet = None
        self.join_column = join_column
        self.merge_cell = merge_cell
        self.merge_cells_id = []
        self.join_column_id = None
        self.max_erow = max_empty_rows
        self.max_ecol = max_empty_columns
        self.verbose = verbose
        self.debug = debug

    def log(self, message, forced=False, severity=4):
        if forced or (self.verbose and severity < 7) or self.debug:
            print('{}\t{}'.format(self.name, message))

    def col_id(self, col_name):
        """Get the number of column that has a given name in the first cell"""
        i = 1
        empty_count = 0
        while empty_count < self.max_ecol:
            value = self.sheet.cell(row=1, column=i).value
            if value == col_name:
                return i
            if value is None:
                empty_count += 1
            i += 1
        raise Exception('Column "{}" not found in file "{}"'.format(col_name, self.name))

    def find_join_id(self):
        self.join_column_id = self.col_id(self.join_column)
        self.log('Found join column "{}" at position #{}'.format(self.join_column, self.join_column_id))

    def find_merge_cells_id(self):
        self.merge_cells_id = []
        for cell in self.merge_cell:
            cid = self.col_id(cell)
            self.merge_cells_id.append(cid)
            self.log('Found merge column "{}" at position #{}'.format(cell, cid))

    def join_row(self, cell_content):
        """Return the row ID of the row containing the cell_content in the join column"""
        i = 1
        empty_count = 0
        while empty_count < self.max_erow:
            value = self.sheet.cell(row=i, column=self.join_column_id).value
            if value == cell_content:
                return i
            if value is None:
                empty_count += 1
            i += 1
        self.log('WARNING, cell with content "{}" not found in file "{}"'.format(cell_content, self.name))

    def export_cells(self, row_id):
        """Export all the merge cells where the join column is valued cell_content"""
        export = []
        if row_id is not None:
            for cell_id in self.merge_cells_id:
                export.append(self.sheet.cell(row=row_id, column=cell_id).value)
            printable = []
            for item in export:
                if item is not None:
                    printable.append(str(item))
                else:
                    printable.append('')
            self.log('Row #{} exported the following cells: {}'.format(row_id, ', '.join(printable)), severity=7)
        else:
            self.log('Attempted export of cells while row_id is None', severity=7)
        return export

    def export_all_cells(self):
        i = 2
        empty_count = 0
        export = []
        while empty_count < self.max_erow:
            value = self.sheet.cell(row=i, column=self.join_column_id).value
            row_export = [value]
            row_export.extend(self.export_cells(i))
            if value is None:
                empty_count += 1
            else:
                export.append(row_export)
            i += 1
        return export

    def absorbe(self, list_of_rows, override=False, highlight=False):
        for row in list_of_rows:
            row_id = self.join_row(row[0])
            if row_id:
                cell_list = row[1:]
                for num, cell in enumerate(cell_list):
                    try:
                        old_value = self.sheet.cell(row=row_id, column=self.merge_cells_id[num]).value
                        if old_value is not None and not override and old_value != cell:
                            self.log('SKIP ROW {} on col "{}" because override attempt ("{}" over "{}")'.format(
                                row_id,
                                self.merge_cell[num],
                                cell,
                                old_value
                            ), forced=True)
                        else:
                            self.sheet.cell(row=row_id, column=self.merge_cells_id[num]).value = cell
                            if highlight:
                                self.sheet.cell(row=row_id, column=self.merge_cells_id[num]).fill = openpyxl.styles.PatternFill(
                                    start_color='FFFF0000',
                                    end_color='FFFF0000',
                                    fill_type='solid'
                                )
                    except IndexError:
                        self.log('ERROR, no merge cell #{}, total merge cells are {}'.format(
                            num,
                            len(self.merge_cells_id)
                        ), forced=True)
                        exit(1)
